@@ -143,9 +143,6 @@ enum {
   ATTR_PROTECTED  = 0x20000000u,
   ATTR_FRAMED     = 0x0010000000000000u,
 
-  GRAPH_MASK      = 0x00000F0000000000u,
-  ATTR_GRAPH_SHIFT = 40,
-
   FONTFAM_MASK    = 0x000F000000000000u,
   ATTR_FONTFAM_SHIFT = 48,
 
@@ -172,11 +169,23 @@ enum {
   TATTR_CLEAR     = 0x4000000000000000u, /* erased / unwritten */
   TATTR_OVERHANG  = 0x0080000000000000u, /* visual double-width overhang */
 
+  TATTR_JOINED    = 0x0000080000000000u, /* LAM/ALEF joined as combining */
+
   DATTR_STARTRUN  = 0x8000000000000000u, /* start of redraw run */
   DATTR_MASK      = TATTR_RIGHTCURS | TATTR_PASCURS | TATTR_ACTCURS
-                    | DATTR_STARTRUN
+                    | DATTR_STARTRUN,
+
+#ifdef configured_glyph_shift
+  GLYPHSHIFT_MASK = 0x0000070000000000u,
+  ATTR_GLYPHSHIFT_SHIFT = 40,
+#define GLYPHSHIFT_MAX (GLYPHSHIFT_MASK >> ATTR_GLYPHSHIFT_SHIFT)
+#else
+  ATTR_GLYPHSHIFT = 0x0000040000000000u, /* alternative font glyph shift */
+#endif
+
   // unassigned bits:
-  // - none
+  //              = 0x0000020000000000u,
+  //              = 0x0000010000000000u,
 };
 
 /* Line attributes.
@@ -561,6 +570,7 @@ struct term {
   bool wide_indic;
   bool wide_extra;
   bool disable_bidi;
+  bool join_lam_alef;
   bool enable_bold_colour;
   bool enable_blink_colour;
   bool readline_mouse_1;
